#ifndef BRUSHINTERFACE_H
#define BRUSHINTERFACE_H

#include <QtPlugin>
#include <QtGlobal>

#include "plugins.h"
#include "tool.h"
#include "../eventcontainer.h"

QT_BEGIN_NAMESPACE
class QImage;
class QPainter;
class QWidget;
class QPainterPath;
class QPoint;
class QRegion;
class QString;
class QStringList;
class QIcon;
class QInputEvent;
class QUuid;
class QWaitCondition;
class QMutex;
QT_END_NAMESPACE

class BrushInterface : public Tool {
	Q_OBJECT
public:
	BrushInterface();
	// some compilers are said to complain about defining virtual functions and
	// not declaring virtual destructors. keep compilers happy.
	// http://doc.qt.nokia.com/4.6/tools-plugandpaint.html
	virtual ~BrushInterface();

	virtual QStringList brushes() const = 0;

	virtual QList<QIcon> icons() const = 0;

	virtual QRegion brushPress(const QUuid &brush, QImage &image,
							 QPainter &painter, const QInputEvent *event) = 0;
	virtual QRegion brushMove(const QUuid &brush, QImage &image,
							QPainter &painter, const QInputEvent *oldEvent,
							const QInputEvent *newEvent) = 0;
	virtual QRegion brushRelease(const QUuid &brush, QImage &image,
							   QPainter &painter, const QInputEvent *event) = 0;

	//FIXME:
	//need to be changed, this is only temp
	QString interfaceVersion() const;

signals:
	void update(QRegion region);
	void beginPainting();
	void endPainting();
};

QT_BEGIN_NAMESPACE
Q_DECLARE_INTERFACE(BrushInterface,
					"com.cybercom.Carto.BrushInterface/0.4.5")
QT_END_NAMESPACE

#endif // BRUSHINTERFACE_H
