#include "layersdock.h"

#include <QVBoxLayout>
#include <QHBoxLayout>

#include <QDebug>

LayersDock::LayersDock(const QString &title, QWidget *parent) :
	QDockWidget(parent),
	m_pLM(NULL)
{
	this->setWindowTitle( title );
	this->setAllowedAreas( Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
	this->setFeatures(QDockWidget::AllDockWidgetFeatures);

	QWidget *item = new QWidget(this);
	item->setAttribute( Qt::WA_DeleteOnClose);

	this->setWidget( item );
	item->setLayout( new QVBoxLayout(item) );
	item->layout()->setMargin(0);

	m_pList = new QListWidget(item);

	item->layout()->addWidget(m_pList);
	connect( m_pList, SIGNAL(itemClicked(QListWidgetItem*)),
			 this, SLOT(itemActivated(QListWidgetItem*)) );

	QWidget *item2 = new QWidget(item);
	item2->setAttribute( Qt::WA_DeleteOnClose);
	item2->setLayout( new QHBoxLayout(item2) );
	item2->layout()->setMargin(0);


	m_pAddButton = new QToolButton(item2);
	m_pRemoveButton = new QToolButton(item2);

	m_pAddButton->setAutoRaise( true );
	m_pRemoveButton->setAutoRaise( true );

	m_pAddAction = new QAction(QIcon(":/toolbar/res/toolbar/list-add.png"),
							   tr("Add Layer"), m_pAddButton);

	m_pRemoveAction = new QAction(
			QIcon(":/toolbar/res/toolbar/list-remove.png"),
			tr("Remove Layer"), m_pRemoveButton);


	m_pAddAction->setEnabled( false );
	m_pRemoveAction->setEnabled( false );

	connect(m_pAddAction, SIGNAL(triggered()), this, SLOT(addLayerPrivate()));
	connect(m_pRemoveAction, SIGNAL(triggered()), this,
			SLOT(removeLayerPrivate()));

	m_pAddButton->setDefaultAction( m_pAddAction );
	m_pRemoveButton->setDefaultAction(m_pRemoveAction);

	item2->layout()->addWidget(m_pAddButton);
	item2->layout()->addWidget(m_pRemoveButton);

	item->layout()->addWidget( item2 );

}

LayersDock::~LayersDock(){
	delete m_pList;
	delete m_pAddAction;
	delete m_pRemoveAction;
	delete m_pAddButton;
	delete m_pRemoveButton;
}
//------------------------------------------------------------------------------
void LayersDock::populateList(const QList<Layer *> &layers){
	m_pList->clear();

	QList<Layer*>::const_iterator i;

	for( i = layers.constBegin(); i != layers.constEnd(); ++i ){
		m_pList->addItem( (*i)->getName() );
	}
}

inline int LayersDock::calculatePos(int pos){
	if( pos == -1)
		pos = m_pList->currentRow();
	return m_pLM->count() - pos - 1;
}

//------------------------------------------------------------------------------
void LayersDock::setLayersManager(LayersManager *lm){
	if( m_pLM )
		m_pLM->disconnect( this );

	m_pLM = lm;

	if( m_pLM ){
		populateList( m_pLM->getLayersList() );

		connect( m_pLM, SIGNAL(destroyed(QObject*)),
				 this, SLOT(childDestroyed(QObject*)) );

		connect( m_pLM, SIGNAL(layerAdded(uint,QString)),
				 this, SLOT(addLayer(uint,QString)) );
		connect( m_pLM, SIGNAL(layerRemoved(uint)),
				 this, SLOT(removeLayer(uint)) );
		connect( m_pLM, SIGNAL(layerRenamed(uint,QString)),
				 this, SLOT(renameLayer(uint,QString)) );

		m_pAddAction->setEnabled( true );

		if( m_pLM->count() > 1)
			m_pRemoveAction->setEnabled( true );
		else
			m_pRemoveAction->setEnabled( false );

		m_pList->setCurrentRow( m_pLM->getActiveLayer() );
	}else{ //if NULL clear and lock dock
		m_pList->clear();
		m_pRemoveAction->setEnabled( false );
		m_pAddAction->setEnabled( false );
	}

}

LayersManager* LayersDock::getLayersManager() const{
	return m_pLM;
}

void LayersDock::setTitle(const QString &title){
	this->setWindowTitle( title );
}

QString LayersDock::getTitle() const {
	return windowTitle();
}

//------------------------------------------------------------------------------
void LayersDock::childDestroyed(QObject *obj){
	if( NULL == obj || obj != m_pLM )
		return;

	m_pLM->disconnect( this );
	m_pLM = NULL;

	m_pList->clear();

	m_pAddAction->setEnabled( false );
	m_pRemoveAction->setEnabled( false );

}

void LayersDock::itemActivated(QListWidgetItem *item){
	if( item == NULL )
		return;

	uint pos = m_pList->row( item );
	m_pLM->setActiveLayer( calculatePos( pos ) );
}

void LayersDock::addLayerPrivate(){
	m_pLM->addLayer( calculatePos() );
}

void LayersDock::removeLayerPrivate(){
	m_pLM->removeLayer( calculatePos() );
}

//------------------------------------------------------------------------------
void LayersDock::addLayer(uint pos, QString name){
	m_pList->insertItem( calculatePos(pos)-1, name );

	if( m_pLM->count() > 1)
		m_pRemoveAction->setEnabled( true );
}

void LayersDock::removeLayer(uint pos){
	QListWidgetItem* item = m_pList->takeItem( calculatePos(pos) + 1 );
	delete item;

	if( m_pLM->count() == 1)
		m_pRemoveAction->setEnabled( false );
}

void LayersDock::renameLayer(uint pos, QString newName){
	m_pList->item( calculatePos(pos) )->setText( newName );
}

