/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the examples of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial Usage
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Commercial License Agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Nokia.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
** If you have questions regarding the use of this file, please contact
** Nokia at qt-info@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/


#include "interfaces/brushinterface.h"
#include "plugindialog.h"

#include <QPluginLoader>
#include <QStringList>
#include <QDir>

#include <QLabel>
#include <QGridLayout>
#include <QPushButton>
#include <QTreeWidget>
#include <QTreeWidgetItem>
#include <QHeaderView>
#include <QFormLayout>

#include <QDebug>

PluginDialog::PluginDialog(const QString &path, const QStringList &fileNames,
						   QWidget *parent) :
	QDialog(parent),
	m_pLabel(new QLabel),
	m_pTreeWidget(new QTreeWidget),
	m_pOkButton(new QPushButton(tr("OK")))
{
	this->setWindowFlags(Qt::Dialog | Qt::WindowCloseButtonHint);
	m_pTreeWidget->setAlternatingRowColors(false);
	m_pTreeWidget->setSelectionMode(QAbstractItemView::NoSelection);
	m_pTreeWidget->setColumnCount(1);
	m_pTreeWidget->header()->hide();

	m_pOkButton->setDefault(true);

	connect(m_pOkButton, SIGNAL(clicked()), this, SLOT(close()));

	m_pGroupBox = new QGroupBox( tr("Plugin info"), this);
	initiateLabels();

	QGridLayout *mainLayout = new QGridLayout;
	mainLayout->setColumnStretch(0, 1);
	mainLayout->setColumnStretch(2, 1);
	mainLayout->addWidget(m_pLabel, 0, 0, 1, 3);
	mainLayout->addWidget(m_pTreeWidget, 1, 0, 1, 3);
	mainLayout->addWidget(m_pGroupBox, 2, 0, 1, 3);
	mainLayout->addWidget(m_pOkButton, 3, 1);
	setLayout(mainLayout);

	m_interfaceIcon.addPixmap(style()->standardPixmap(QStyle::SP_DirOpenIcon),
							  QIcon::Normal, QIcon::On);
	m_interfaceIcon.addPixmap(
			style()->standardPixmap(QStyle::SP_DirClosedIcon),
			QIcon::Normal, QIcon::Off);

	setWindowTitle(tr("Plugin Information"));

	connect( m_pTreeWidget, SIGNAL(itemActivated(QTreeWidgetItem*,int)),
			 this, SLOT(loadInfo(QTreeWidgetItem*,int)));
	connect( m_pTreeWidget, SIGNAL(itemClicked(QTreeWidgetItem*,int)),
			 this, SLOT(loadInfo(QTreeWidgetItem*,int)));
	findPlugins(path, fileNames);
}

PluginDialog::~PluginDialog(){
	delete m_pOkButton;
	delete m_pLabel;
	delete m_pTreeWidget;

	delete m_pLabelShortNameTitle;
	delete m_pLabelVersionStringTitle;
	delete m_pLabelDescriptionTitle;
	delete m_pLabelAuthorTitle;
	delete m_pLabelAuthorEmailTitle;
	delete m_pLabelCopyrightTitle;
	delete m_pLabelHomePageTitle;

	delete m_pLabelShortName;
	delete m_pLabelVersionString;
	delete m_pLabelDescription;
	delete m_pLabelAuthor;
	delete m_pLabelAuthorEmail;
	delete m_pLabelCopyright;
	delete m_pLabelHomePage;

	delete m_pGroupBox;
}

void PluginDialog::initiateLabels(){
	m_pLabelShortNameTitle = new QLabel(tr("Short Name:"), this);
	m_pLabelVersionStringTitle = new QLabel(tr("Version:"), this);
	m_pLabelDescriptionTitle = new QLabel(tr("Description:"), this);
	m_pLabelAuthorTitle = new QLabel(tr("Author:"), this);
	m_pLabelAuthorEmailTitle = new QLabel(tr("Author email:"), this);
	m_pLabelCopyrightTitle = new QLabel(tr("Copyright:"), this);
	m_pLabelHomePageTitle = new QLabel(tr("Homepage:"), this);

	m_pLabelShortName = new QLabel(this);
	m_pLabelVersionString = new QLabel(this);
	m_pLabelDescription = new QLabel(this);
	m_pLabelAuthor = new QLabel(this);
	m_pLabelAuthorEmail = new QLabel(this);
	m_pLabelCopyright = new QLabel(this);
	m_pLabelHomePage = new QLabel(this);

	m_pLabelAuthorEmail->setTextInteractionFlags( Qt::TextBrowserInteraction );
	m_pLabelAuthorEmail->setOpenExternalLinks( true );
	m_pLabelHomePage->setTextInteractionFlags( Qt::TextBrowserInteraction );
	m_pLabelHomePage->setOpenExternalLinks( true );

	QFormLayout *layout = new QFormLayout(this);
	m_pGroupBox->setLayout( layout );

	layout->addRow(m_pLabelShortNameTitle, m_pLabelShortName);
	layout->addRow(m_pLabelVersionStringTitle, m_pLabelVersionString);
	layout->addRow(m_pLabelDescriptionTitle, m_pLabelDescription);
	layout->addRow(m_pLabelAuthorTitle, m_pLabelAuthor);
	layout->addRow(m_pLabelAuthorEmailTitle, m_pLabelAuthorEmail);
	layout->addRow(m_pLabelCopyrightTitle, m_pLabelCopyright);
	layout->addRow(m_pLabelHomePageTitle, m_pLabelHomePage);
}

void PluginDialog::loadInfo(QTreeWidgetItem *item, int column){
	if( item == NULL )
		return;

	if( item->data(0,32).isValid() ){
		QObject *ptr = qvariant_cast<QObject*>( item->data(0,32) );
		Plugin *plugin = dynamic_cast<Plugin *>( ptr );
		if( plugin ){
			m_pGroupBox->setTitle( plugin->metaObject()->className() );
			m_pLabelAuthor->setText( plugin->getAuthorInfo() );
			m_pLabelHomePage->setText( "<a href=\"mailto:" +
									   plugin->getAuthorEmailInfo() +
									   "\">" + plugin->getAuthorEmailInfo() +
									   "</a>");
			m_pLabelCopyright->setText( plugin->getCopyrightInfo());
			m_pLabelDescription->setText( plugin->getDescriptionInfo() );
			m_pLabelHomePage->setText( "<a href=\"" +
									   plugin->getHomepageInfo() +
									   "\">" + plugin->getHomepageInfo() +
									   "</a>");
			m_pLabelShortName->setText( plugin->getShortNameInfo() );
			m_pLabelVersionString->setText( plugin->getVersionStringInfo() );
		}
	}
}

void PluginDialog::findPlugins(const QString &path,
							   const QStringList &fileNames)
{
	m_pLabel->setText(tr("Carto found the following plugins\n"
						"(looked in %1):").arg(QDir::toNativeSeparators(path)));

	const QDir dir(path);

	foreach (QObject *plugin, QPluginLoader::staticInstances())
		populateTreeWidget(plugin, tr("%1 (Static Plugin)")
								   .arg(plugin->metaObject()->className()));

	foreach (QString fileName, fileNames) {
		QPluginLoader loader(dir.absoluteFilePath(fileName));
		QObject *plugin = loader.instance();
		if (plugin)
			populateTreeWidget(plugin, fileName);
	}
}

void PluginDialog::populateTreeWidget(QObject *plugin, const QString &text) {
	QTreeWidgetItem *pluginItem = new QTreeWidgetItem(m_pTreeWidget);
	pluginItem->setText(0, text);
	m_pTreeWidget->setItemExpanded(pluginItem, true);

	QFont boldFont = pluginItem->font(0);
	boldFont.setBold(true);
	pluginItem->setFont(0, boldFont);

	if (plugin){
		pluginItem->setData(0, 32, qVariantFromValue(plugin) );
		BrushInterface *iBrush = qobject_cast<BrushInterface *>(plugin);
		if (iBrush){
			addItems(pluginItem,
					 "BrushInterface " + iBrush->interfaceVersion().toLatin1(),
					 iBrush->brushes(), iBrush->uuids(), iBrush->icons());
		}
	}else{
		delete pluginItem;
	}
}

void PluginDialog::addItems(QTreeWidgetItem *pluginItem,
							const QString interfaceName,
							const QStringList &tools,
							const QList<QUuid> &uuids,
							const QList<QIcon> &icons)
{
	QTreeWidgetItem *interfaceItem = new QTreeWidgetItem(pluginItem);
	interfaceItem->setText(0, interfaceName);
	interfaceItem->setIcon(0, m_interfaceIcon);

	for( int i = 0; i < tools.size(); i++){
		QString tool = tools.at(i);
		if( tool.endsWith("...") )
			tool.chop(3);
		QTreeWidgetItem *toolItem = new QTreeWidgetItem(interfaceItem);
		toolItem->setText(0, tool);
		toolItem->setIcon(0, icons.at(i));
		toolItem->setToolTip(0, uuids.at(i).toString());
	}
}
