#ifndef TOOLSSETTINGS_H
#define TOOLSSETTINGS_H

#include <QObject>
#include "interfaces/plugins.h"
#include "interfaces/brushinterface.h"
#include "definitions.h"


#include <QMap>
#include <QVariant>
#include <QVector>
#include <QString>
#include <QVariantMap>
#include <QUuid>

#include <QSqlDatabase>
#include <QSqlError>
#include <QSqlQuery>

/*
 this is singleton pattern class
 */

class ToolsSettings : public QObject {
    Q_OBJECT
	Q_DISABLE_COPY(ToolsSettings)
private:
	explicit ToolsSettings(QObject *parent = 0);

	QUuid m_tool;
	Plugin *m_pPlugin;

	/*SQL*/
	bool createConnection();
	bool initiateDatabase();


	QString m_memoryDBconnection;

public:
	~ToolsSettings();
	static ToolsSettings& getInstance(){
		static ToolsSettings instance;
		return instance;
	}

	QVariant getSetting(const QString &tool, int device, uint feature) const;
	bool setSetting(const QString &tool, int device,
					uint feature, QVariant value);

	QVariant getMemorySetting(uint feature, int device = 0) const;
	bool setMemorySetting(uint feature, int device,
						  QVariant value);
	bool wipeSettings(bool safe = false);

	const QUuid* bindTool() const;
	Plugin** bindInterface();

signals:
	void toolChanged(QUuid tool);

public slots:
	void setTool(Plugin *_plugin, QUuid _tool);
};

#endif // TOOLSSETTINGS_H
