#include "mainwindow.h"
#include "../plugindialog.h"
#include "ui_mainwindow.h"

#include "../interfaces/brushinterface.h"
#include "../interfaces/plugins.h"
#include "../interfaces/tool.h"

#include "about.h"
#include "newdialog.h"

#include "../toolssettings.h"

#include <QColorDialog>
#include <QPluginLoader>
#include <QToolButton>
#include <QVBoxLayout>
#include <QGridLayout>
#include <QHBoxLayout>
#include <QToolBar>
#include <QDockWidget>
#include <QSpinBox>
#include <QLabel>
#include <QCheckBox>
#include <QPushButton>
#include <QLayoutItem>
#include <QObjectList>
#include <QSlider>
#include <QUndoView>
#include <QUndoGroup>
#include <QUndoStack>
#include <QFileDialog>

#include <QDebug>


MainWindow::MainWindow(QWidget *parent) :
					   QMainWindow(parent),
					   ui(new Ui::MainWindow)
{
	ui->setupUi(this);
	ui->mdiArea->setDocumentMode(true);

	m_pWindowMapper = new QSignalMapper(this);
	m_pWindowsGroup = new QActionGroup(ui->menuWindow);

	connect(m_pWindowMapper, SIGNAL(mapped(QWidget*)),
		   this, SLOT(setActiveSubWindow(QWidget*)));

	m_pDocksGroup = new QActionGroup(ui->menuDocks);


	this->setWindowTitle(qApp->applicationName());

//----------------------
	ui->dockWidget_toolbox->setWindowTitle(tr("ToolBox"));

	QVBoxLayout *layout = new QVBoxLayout(ui->dockWidget_toolbox->widget());

	ui->dockWidget_toolbox->widget()->setLayout(layout);
	ui->dockWidgetContents->layout()->setAlignment( Qt::AlignTop );
	ui->dockWidget_toolbox->setEnabled( false );
//----------------------
	QHBoxLayout *layoutSettings = new QHBoxLayout(ui->dockWidgetContents_3);
	ui->dockWidgetContents_3->setLayout(layoutSettings);
//----------------------
	ui->actionNew->setShortcut(QKeySequence::New);
	ui->actionOpen->setShortcut(QKeySequence::Open);
	ui->actionSave->setShortcut(QKeySequence::Save);
	ui->actionSave_as->setShortcut(QKeySequence::SaveAs);
	ui->actionClose->setShortcut(QKeySequence::Close);
#ifdef Q_WS_WIN
	ui->actionExit->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_Q));
#else
	ui->actionExit->setShortcut(QKeySequence::Quit);
#endif
	ui->actionUndo->setShortcut(QKeySequence::Undo);
	ui->actionRedo->setShortcut(QKeySequence::Redo);
	ui->actionCopy->setShortcut(QKeySequence::Copy);
	ui->actionPaste->setShortcut(QKeySequence::Paste);
	ui->actionCut->setShortcut(QKeySequence::Cut);
//----------------------
	connect(ui->aboutPluginsAct, SIGNAL(triggered()), this,
			SLOT(aboutPlugins()));
	connect(ui->actionQt, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
	connect(ui->actionCarto, SIGNAL(triggered()), this, SLOT(showAbout()));

	connect(ui->actionNew, SIGNAL(triggered()), this, SLOT(showNewDialog()));
	connect(ui->actionClose, SIGNAL(triggered()), this, SLOT(closeActive()));
	connect(ui->actionExit, SIGNAL(triggered()), this, SLOT(close()));

	connect(ui->actionOpen, SIGNAL(triggered()), this, SLOT(openFile()));

	connect(ui->actionCascade, SIGNAL(triggered()), ui->mdiArea,
			SLOT(cascadeSubWindows()));
	connect(ui->actionTile, SIGNAL(triggered()), ui->mdiArea,
			SLOT(tileSubWindows()));


	ui->actionSave->setEnabled( false );
	ui->actionSave_as->setEnabled( false );
	connect(ui->actionSave,SIGNAL(triggered()),
			this, SLOT(save()));
	connect(ui->actionSave_as, SIGNAL(triggered()),
			this, SLOT(saveAs()));
	connect(ui->actionUndo, SIGNAL(triggered()), this, SLOT(undo()));
	connect(ui->actionRedo, SIGNAL(triggered()), this, SLOT(redo()));

#ifndef QT_NO_DEBUG
	m_debugMenu = menuBar()->addMenu(tr("Debug"));
	m_repaintPaintArea = new QAction(tr("Repaint current PaintArea"), this);
	m_debugMenu->addAction(m_repaintPaintArea);
	connect(m_repaintPaintArea, SIGNAL(triggered()), this,
			SLOT(repaintCurrent()));
#endif

	//e.g. for MainWindow title change ;)
	connect(ui->mdiArea, SIGNAL(subWindowActivated(QMdiSubWindow*)),this,
			SLOT(subWindowActiv(QMdiSubWindow*)));
	loadPlugins();

	//-----------------------------------
	m_pUndoDock = new QDockWidget(tr("Command List"), this);
	m_pUndoDock->setWidget(new QUndoView(m_pUndoDock));
	m_pUndoDock->setObjectName("undoDock");
	this->addDockWidget(Qt::RightDockWidgetArea, m_pUndoDock);
	m_pUndoDock->show(); //if no entry in saved settings
	//-----------------------------------
	m_pLayersDock = new LayersDock(tr("Layers"), this);
	m_pLayersDock->setObjectName("layersDock");
	this->addDockWidget(Qt::RightDockWidgetArea, m_pLayersDock);
	m_pLayersDock->show(); //if no entry in saved settings
	//-----------------------------------

	readSettings();

	//-----------------------------------
	ToolsSettings::getInstance();
	//-----------------------------------

	initaiteDockMenu();
#ifndef Q_OS_MAC
	initiateActionWigets();
#endif
}

MainWindow::~MainWindow() {
	delete ui;
	delete m_pWindowMapper;
	delete m_pWindowsGroup;
	delete m_pDocksGroup;
	delete m_pUndoDock->widget();
	delete m_pUndoDock;
	delete m_pLayersDock;
	delete m_pEditMenuWA;
}

/*---------------------------------------------------------------------------*/
void MainWindow::closeEvent(QCloseEvent *event){
	ui->mdiArea->closeAllSubWindows();
	if( !ui->mdiArea->subWindowList().isEmpty() ) {
		event->ignore();
	} else {
		writeSettings();
		event->accept();
	}
}

void MainWindow::changeEvent(QEvent *event){
	if( event->type() == QEvent::ModifiedChange ){
		QMdiSubWindow *window = getSubWindow();
		if( window && !isWindowModified() && window->isWindowModified()){
			changeWindowModified(window->isWindowModified());
		}
	}
}

/*---------------------------------------------------------------------------*/
void MainWindow::initaiteDockMenu(){
	m_pDocksGroup->setExclusive( false );

	//QAction *action = ui->menuDocks->addAction(tr("Settings"));
	QAction *action = ui->dockWidget_3->toggleViewAction();
	m_pDocksGroup->addAction( action );


	//action = ui->menuDocks->addAction(tr("Toolbox"));
	action = ui->dockWidget_toolbox->toggleViewAction();
	m_pDocksGroup->addAction( action );


	//action = ui->menuDocks->addAction(tr("Layers"));
	action = m_pLayersDock->toggleViewAction();
	m_pDocksGroup->addAction( action );

	//action = ui->menuDocks->addAction(tr("Undo"));
	action = m_pUndoDock->toggleViewAction();
	m_pDocksGroup->addAction( action );

	ui->menuDocks->addActions(m_pDocksGroup->actions());

}

void MainWindow::initiateActionWigets(){
	m_pEditMenuWA = new QWidgetAction(ui->menuEdit);

	QWidget *area = new QWidget(ui->menuEdit);

	QHBoxLayout *layout = new QHBoxLayout(area);
	layout->setContentsMargins(33,0,0,0);

	area->setLayout(layout);

	layout->addWidget(new QLabel(tr("Edit:")),100);

	QToolButton *qtb = new QToolButton(area);
	qtb->setAutoRaise(true);
	qtb->setDefaultAction(ui->actionCopy);
	layout->addWidget(qtb);

	qtb = new QToolButton(area);
	qtb->setAutoRaise(true);
	qtb->setDefaultAction(ui->actionPaste);
	layout->addWidget(qtb);

	qtb = new QToolButton(area);
	qtb->setAutoRaise(true);
	qtb->setDefaultAction(ui->actionCut);
	layout->addWidget(qtb);


	m_pEditMenuWA->setDefaultWidget(area);

	ui->menuEdit->addAction( m_pEditMenuWA );

	ui->menuEdit->removeAction(ui->actionCopy);
	ui->menuEdit->removeAction(ui->actionCut);
	ui->menuEdit->removeAction(ui->actionPaste);
}

/*---------------------------------------------------------------------------*/

void MainWindow::eraseLayout(QLayout *layout){
	if( !layout )
		return;

	QLayoutItem *item;

	while ((item = layout->takeAt(0))){
		if(item->widget() == m_pAlpha)
			m_pAlpha = NULL;
		if(item->widget() == m_pColor)
			m_pColor = NULL;

		delete item->widget();
		delete item;
	}

}

void MainWindow::loadPlugins() {
	foreach (QObject *plugin, QPluginLoader::staticInstances()){
		produceToolButtons(plugin);
	}

	m_pluginsDir = QDir(qApp->applicationDirPath());

#if defined(Q_OS_WIN)
	if (m_pluginsDir.dirName().toLower() == "debug" ||
		m_pluginsDir.dirName().toLower() == "release")
		m_pluginsDir.cdUp();
#elif defined(Q_OS_MAC)
	if (m_pluginsDir.dirName() == "MacOS") {
		m_pluginsDir.cdUp();
		m_pluginsDir.cdUp();
		m_pluginsDir.cdUp();
	}
#endif
	m_pluginsDir.cd("plugins");

	foreach (QString fileName, m_pluginsDir.entryList(QDir::Files)) {
		QPluginLoader loader(m_pluginsDir.absoluteFilePath( fileName ));
		QObject *plugin = loader.instance();
		if( plugin ){
			produceToolButtons( plugin );
			m_pluginFileNames += fileName;
		}
	}

}

void MainWindow::populateSettingsDock(QObject *_plugin, const QString &uuid ){
	QHBoxLayout *layout =
			qobject_cast<QHBoxLayout*>( ui->dockWidgetContents_3->layout() );

	eraseLayout( layout );


	if( layout == NULL )
		return;

	Tool *tool = dynamic_cast<Tool*>(_plugin);

	if( tool == NULL )
		return;

	ToolsSettings::getInstance().wipeSettings(true);

	QVariant savedValue;
	if( tool->testFeature(Tool::SizeChange) ){
		layout->addWidget( new QLabel(tr("Thickness:")) );

		QSpinBox *spinBox = new QSpinBox(ui->dockWidgetContents_3);
		spinBox->setMinimum(1);
		spinBox->setMaximum(1024);
		savedValue = ToolsSettings::getInstance().getSetting(
				uuid, 0, Tool::SizeChange);

		if( savedValue.isValid() )
			spinBox->setValue( savedValue.toInt() );
		else
			spinBox->setValue(3);

		ToolsSettings::getInstance().setMemorySetting(Tool::SizeChange, 0,
												 spinBox->value());

		layout->addWidget(spinBox);
		connect(spinBox, SIGNAL(valueChanged(int)), this,
				SLOT(changeSize(int)));

	}


	if( tool->testFeature(Tool::AntiAliasing) ){
		QCheckBox *chBox = new QCheckBox(tr("Antialiasing:"),
										 ui->dockWidgetContents_3);
		savedValue = ToolsSettings::getInstance().getSetting(
				uuid, 0, Tool::AntiAliasing);

		if( savedValue.isValid() )
			chBox->setChecked( savedValue.toBool() );
		else
			chBox->setChecked(true);

		ToolsSettings::getInstance().setMemorySetting(Tool::AntiAliasing, 0,
													  chBox->isChecked());

		layout->addWidget(chBox);
		connect(chBox, SIGNAL(stateChanged(int)), this, SLOT(changeAA(int)));
	}

	if( tool->testFeature(Tool::ColorChange) ){
		QLabel *label = new QLabel(tr("Color"), ui->dockWidgetContents_3);


		savedValue = ToolsSettings::getInstance().getSetting(
						uuid, 0, Tool::ColorChange);
		QColor color;
		if( savedValue.isValid() ){
			color = savedValue.value<QColor>();
			label->setStyleSheet( "background-color: rgba( " +
								  QString::number(color.red()) + ", " +
								  QString::number(color.green()) + ", " +
								  QString::number(color.blue()) + ", " +
								  QString::number(color.alpha()) + ");" );
		}else{
			color = QColor(0,0,255);
			label->setStyleSheet("background-color: rgb(0,0,255);");
		}

		ToolsSettings::getInstance().setMemorySetting(Tool::ColorChange, 0,
													  color);

		layout->addWidget(label);
		m_pColor = label;

		QPushButton *button = new QPushButton(tr("Choose Color"),
											  ui->dockWidgetContents_3);
		layout->addWidget(button);
		connect(button, SIGNAL(clicked()), this, SLOT(changeColor()));

	}

	if( tool->testFeature(Tool::AlphaChange) ){
		QLabel *label = new QLabel(tr("Alpha"), ui->dockWidgetContents_3);

		layout->addWidget(label);

		QSlider *slider = new QSlider(Qt::Horizontal,ui->dockWidgetContents_3);
		slider->setMaximum(255);
		slider->setMinimum(0);
		slider->setPageStep(10);
		slider->setTracking(false);

		savedValue = ToolsSettings::getInstance().getSetting(
				uuid, 0, Tool::AlphaChange);

		if( savedValue.isValid() )
			slider->setValue( savedValue.toInt() );
		else
			slider->setValue(255);

		ToolsSettings::getInstance().setMemorySetting(Tool::AlphaChange, 0,
													  slider->value());

		m_pAlpha = slider;
		layout->addWidget(slider);
		connect(slider, SIGNAL(valueChanged(int)),
				this, SLOT(changeAlpha(int)));
	}

	layout->addStretch();

}


void MainWindow::produceToolButtons(QObject *_plugin) {
	Tool *plugin = dynamic_cast<Tool*>(_plugin);
	if( plugin ){
		BrushInterface *iBrush;
		switch( plugin->type() ){
		case Tool::Brush:
			iBrush = qobject_cast<BrushInterface*>(_plugin);
			if ( iBrush )
				addToolButton(_plugin, iBrush->uuids(),
							  iBrush->brushes(), iBrush->icons(),
							  SLOT(changeBrush()));
			break;
		case Tool::Shape:
			qWarning()<<"Shape plugin currently is not supported";
			break;
		default:
			qWarning()<<"Unknow plugin type";
		}
	}
}

void MainWindow::addToolButton(QObject *plugin, const QList<QUuid> &uuids,
							   const QStringList &texts,
							   const QList<QIcon> &icons,
							   const char *member) {
#ifndef QT_NO_DEBUG
	qDebug() << "addToolButton";
	qDebug()<< uuids << "\n" << texts;
#endif
	QLayout *layout = ui->dockWidget_toolbox->widget()->layout();

	//QToolBar *qtb = new QToolBar();


	for( int i = 0; i < texts.size(); i++ ){
		QToolButton *button = new QToolButton(ui->dockWidget_toolbox->widget());
		button->setIcon(icons.at(i));
		button->setText(texts.at(i));
		button->setToolTip(texts.at(i));
		button->setAutoRaise(true);
		button->setCheckable(true);
		button->setAutoExclusive(true);
		//FIXME
		//how to add to QVariant own types??
		button->setProperty("plugin", qVariantFromValue(plugin));
		button->setProperty("uuid", uuids.at(i).toString() );
		connect(button, SIGNAL(clicked()), this, member);

		layout->addWidget(button);

	}
	//layout->addWidget(qtb);
	//ui->dockWidget_toolbox->setWidget(qtb);
}

void MainWindow::writeSettings(){
	QSettings settings;
	settings.setIniCodec("UTF-8");
	settings.beginGroup("MainWindow");
;
	settings.setValue("windowState",saveState());
	settings.setValue("geometry", saveGeometry());

	settings.endGroup();
}

void MainWindow::readSettings(){
	QSettings settings;
	settings.setIniCodec("UTF-8");
	settings.beginGroup("MainWindow");

	restoreState( settings.value("windowState").toByteArray());
	restoreGeometry( settings.value("geometry").toByteArray() );
	settings.endGroup();
}

MdiChild* MainWindow::getMdiChild(QMdiSubWindow *window) const{
	if( window )
		return qobject_cast<MdiChild*>(window->widget());
	return NULL;
}

QMdiSubWindow* MainWindow::getSubWindow() const{
	QMdiSubWindow *window = ui->mdiArea->activeSubWindow();
	if( !window )
		window = ui->mdiArea->currentSubWindow();

	return window;
}

QAction* MainWindow::getActiveActionWindow() const{
	QWidget* window = qobject_cast<QWidget *>( this->getSubWindow() );
	if( !window )
		return NULL;

	QAction *action =
			qobject_cast<QAction*>( m_pWindowMapper->mapping(window) );
	return action;
}

/*---------------------------------------------------------------------------*/
void MainWindow::aboutPlugins() {
	PluginDialog dialog(m_pluginsDir.path(), m_pluginFileNames, this);
	dialog.exec();
}

/*---------------------------------------------------------------------------*/

void MainWindow::showAbout(){
	About *dialog = new About(this);
	dialog->exec();
	delete dialog;
}

void MainWindow::showNewDialog(){
	NewDialog *dialog = new NewDialog(this);
	connect(dialog, SIGNAL(createNew(QVariantMap)), this,
			SLOT(createNew(QVariantMap)));
	dialog->exec();
	this->disconnect(dialog);
	delete dialog;
}

void MainWindow::createNew(QVariantMap data){
	MdiChild *child = new MdiChild(ui->mdiArea, &data);

	QMdiSubWindow *window = ui->mdiArea->addSubWindow(child);

	child->show();

	connect(child, SIGNAL(closing(MdiChild*)),
			this, SLOT(prepareCloseActive(MdiChild*)));
	connect(child, SIGNAL(windowTitleChanged(QString)),
			this, SLOT(changeWindowTitle(QString)));
	connect(child, SIGNAL(windowModifiedChanged(bool)),
			this, SLOT(changeWindowModified(bool)));


	QAction *action = ui->menuWindow->addAction(child->getTitle());
	m_pWindowsGroup->addAction( action );

	action->setCheckable( true );
	action->setChecked( window == getSubWindow() );
	connect(action, SIGNAL(triggered()), m_pWindowMapper, SLOT(map()));
	m_pWindowMapper->setMapping( action, window );

	ui->dockWidget_toolbox->setEnabled( true );

	ui->actionSave->setEnabled( true );
	ui->actionSave_as->setEnabled( true );

	if( ui->mdiArea->subWindowList().count() == 1 )
		delete m_pUndoDock->widget();

	m_pUndoDock->setWidget( child->getLayersManager()->getUndoView() );

	QUndoStack* ustack = child->getLayersManager()->getUndoView()->stack();
	if( ustack ){
		connect(ustack, SIGNAL(canRedoChanged(bool)), ui->actionRedo,
				SLOT(setEnabled(bool)));
		connect(ustack, SIGNAL(canUndoChanged(bool)), ui->actionUndo,
				SLOT(setEnabled(bool)));
	}
	QUndoGroup* ugroup = child->getLayersManager()->getUndoView()->group();
	if( ugroup ){
		connect(ugroup, SIGNAL(canRedoChanged(bool)), ui->actionRedo,
				SLOT(setEnabled(bool)));
		connect(ugroup, SIGNAL(canUndoChanged(bool)), ui->actionUndo,
				SLOT(setEnabled(bool)));
	}

	m_pLayersDock->setLayersManager( child->getLayersManager() );

	if( ui->mdiArea->subWindowList().count() == 1 &&
		(*ToolsSettings::getInstance().bindInterface()) &&
		ToolsSettings::getInstance().bindTool()->isNull() != true ){
		populateSettingsDock( (*ToolsSettings::getInstance().bindInterface()),
						ToolsSettings::getInstance().bindTool()->toString() );

	}

}

/*---------------------------------------------------------------------------*/

void MainWindow::subWindowActiv(QMdiSubWindow *mdi){
	if( ui->mdiArea->subWindowList().count() == 0){
		changeWindowTitle("");
		setWindowModified(false);
		ui->dockWidget_toolbox->setEnabled( false );
		eraseLayout( ui->dockWidgetContents_3->layout() );
		ui->actionSave->setEnabled( false );
		ui->actionSave_as->setEnabled( false );
		return;
	}

	if( mdi == NULL ){
		return;
	}

	changeWindowTitle( mdi->windowTitle() );
	changeWindowModified( mdi->isWindowModified() );

	QAction *action = getActiveActionWindow();

	if( action )
		action->setChecked(true);


	if( ui->mdiArea->subWindowList().count() == 1 ){
		return;}
	MdiChild *child = getMdiChild( mdi );
	if( child == NULL )
		return;
	LayersManager *lm = child->getLayersManager();

	lm->restart();

	m_pUndoDock->setWidget( lm->getUndoView() );

	//------------
	if( (lm->getUndoView()->stack() && lm->getUndoView()->stack()->canRedo())||
		(lm->getUndoView()->group() && lm->getUndoView()->group()->canRedo()))
		ui->actionRedo->setEnabled( true );
	else
		ui->actionRedo->setEnabled( false );

	if( (lm->getUndoView()->stack() && lm->getUndoView()->stack()->canUndo())||
		(lm->getUndoView()->group() && lm->getUndoView()->group()->canUndo()))
		ui->actionUndo->setEnabled( true );
	else
		ui->actionUndo->setEnabled( false );


}

void MainWindow::changeWindowTitle(QString title){
	if( title.isEmpty() )
		title = qApp->applicationName();
	else{
		QAction *action = getActiveActionWindow();
		if( action ){
			QString title_copy = title;
			action->setText( title_copy.replace("[*]","") );
		}
		title += " - " + qApp->applicationName();
	}
	this->setWindowTitle(title);
}

void MainWindow::changeWindowModified(bool _state){
	if( isWindowModified() == _state )
		return;

	this->setWindowModified(_state);
	QAction *action = getActiveActionWindow();
	if( action ){
		if( _state ){
			if( action->text().left(1) != "*" )
				action->setText("*" + action->text());
		}else
			action->setText(action->text().replace("*",""));
	}
}

void MainWindow::setActiveSubWindow(QWidget *window){
	if ( window )
		ui->mdiArea->setActiveSubWindow(
				qobject_cast<QMdiSubWindow *>(window));

}

void MainWindow::closeActive(){
	QMdiSubWindow *sub = this->getSubWindow();
	if( sub )
		sub->close();
}

void MainWindow::prepareCloseActive(MdiChild *child){
	if( !child )
		return;
	//here we are disconnecting everything from closing subwindow
	this->disconnect(child);
	child->disconnect(this);

	QWidget* window = qobject_cast<QWidget *>( this->getSubWindow() );

	if( !window ){
		return;
	}

	QObject *actionObject =  m_pWindowMapper->mapping(window);
	m_pWindowMapper->removeMappings( actionObject );

	QAction *action = qobject_cast<QAction*>(actionObject);
	if( action ){
		m_pWindowsGroup->removeAction( action );
		ui->menuWindow->removeAction( action );
	}

	child->prepareToClose(true);
}

void MainWindow::changeBrush() {
	QObject *qtb = sender();

	qDebug()<< "changeBrush =" << qobject_cast<QToolButton*>(qtb)->text()
			<< qtb->property("uuid").toString();

	QObject *ptr = qvariant_cast<QObject*>( qtb->property("plugin"));
	Plugin *plugin = dynamic_cast<Plugin *>( ptr );

	const QUuid uuid( qtb->property("uuid").toString() );

	if( plugin ){
		ToolsSettings::getInstance().setTool(plugin, uuid);
		populateSettingsDock( ptr, qtb->property("uuid").toString() );
	}
}

//-----------------------------------------------------------------------------
void MainWindow::changeColor(){
	QVariant returnVal;
	QColor prevColor;

	returnVal =
			ToolsSettings::getInstance().getMemorySetting(Tool::ColorChange,
														  0);
	if( returnVal.isValid() )
		prevColor = returnVal.value<QColor>();
	else
		prevColor = QColor(Qt::white);

	returnVal =
			ToolsSettings::getInstance().getMemorySetting(Tool::AlphaChange,
														  0);
	if( returnVal.isValid() )
		prevColor.setAlpha( returnVal.toInt() );


	QColor color = QColorDialog::getColor(prevColor, this,
										  tr("Choose color"),
										  QColorDialog::ShowAlphaChannel);

	if( color.isValid() == false )
		return;

	ToolsSettings::getInstance().setMemorySetting(Tool::ColorChange, 0,
												  color);
	ToolsSettings::getInstance().setSetting("", 0, Tool::ColorChange,
											color);

	if( m_pAlpha ){
		QSlider* slider = dynamic_cast<QSlider*>(m_pAlpha);
		if( slider && slider->value() != color.alpha() )
			slider->setValue( color.alpha() );
	}

	if( m_pColor ){
		m_pColor->setStyleSheet("background-color: rgb(" +
								QString::number(color.red()) +
								"," + QString::number(color.green()) + "," +
								QString::number(color.blue()) +
								");");
	}
}

void MainWindow::changeAA(int value){
	ToolsSettings::getInstance().setMemorySetting(Tool::AntiAliasing, 0,
												  static_cast<bool>(value) );
	ToolsSettings::getInstance().setSetting("", 0, Tool::AntiAliasing,
											static_cast<bool>(value) );
}

void MainWindow::changeAlpha(int value){
	ToolsSettings::getInstance().setMemorySetting(Tool::AlphaChange, 0,
												  value );
	ToolsSettings::getInstance().setSetting("", 0, Tool::AlphaChange,
											value );
}

void MainWindow::changeSize(int value){
	ToolsSettings::getInstance().setMemorySetting(Tool::SizeChange, 0,
												  value );
	ToolsSettings::getInstance().setSetting("", 0, Tool::SizeChange,
											value );
}

//-----------------------------------------------------------------------------

void MainWindow::openFile(){
	QString fileName = QFileDialog::getOpenFileName(this, tr("Open image"),
													QString(),
						tr("Images (*.png *.gif *.jpg);;PNG file (*.png);;"
						   "JPG file (*.jpg);;GIF file (*.gif);;"
						   "All files (*) (*.*);;"));
	if( fileName.isEmpty() )
		return;

	QVariantMap vaMap;
	vaMap.insert("open", true);
	vaMap.insert("fileName", fileName);
	this->createNew(vaMap);
}


void MainWindow::save(){
	MdiChild* child = getMdiChild( this->getSubWindow() );
	if( child )
		child->save();
}

void MainWindow::saveAs(){
	MdiChild* child = getMdiChild( this->getSubWindow() );
	if( child )
		child->save( true );
}


#ifndef QT_NO_DEBUG
void MainWindow::repaintCurrent() {
	qDebug() << "Debug: repainting current PaintArea";
	MdiChild *child = getMdiChild(getSubWindow());

	if ( child ) {
		LayersManager *lm = child->getLayersManager();
		lm->update();
		lm->repaint();
	}
}
#endif

void MainWindow::on_actionWindowed_mode_triggered(){
	ui->mdiArea->setViewMode(QMdiArea::SubWindowView);
}

void MainWindow::on_actionTabs_mode_triggered(){
	ui->mdiArea->setViewMode(QMdiArea::TabbedView);
}

void MainWindow::undo(){
	MdiChild* child = getMdiChild(getSubWindow());
	if (child) {
		child->getLayersManager()->undo();
	}
}

void MainWindow::redo(){
	MdiChild* child = getMdiChild(getSubWindow());
	if (child) {
		child->getLayersManager()->redo();
	}
}
