#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QDir>
#include <QStringList>
#include <QVariant>
#include <QMdiSubWindow>
#include <QCloseEvent>
#include <QEvent>
#include <QSignalMapper>
#include <QActionGroup>
#include <QUuid>

#include <QSettings>


#include <QWidgetAction>

#include "../mdichild.h"
#include "../layersdock.h"
#include "../layersmanager.h"

namespace Ui {
	class MainWindow;
}

class MainWindow : public QMainWindow {
	Q_OBJECT

public:
	explicit MainWindow(QWidget *parent = 0);
	~MainWindow();
protected:
	void closeEvent(QCloseEvent *event);
	void changeEvent(QEvent *event);
private:
	Ui::MainWindow *ui;

	QSignalMapper *m_pWindowMapper;
	QActionGroup *m_pWindowsGroup;

	QActionGroup *m_pDocksGroup;

//-----------------------------------
	QDockWidget* m_pUndoDock;
	LayersDock *m_pLayersDock;
//-----------------------------------
	QWidgetAction *m_pEditMenuWA;
//-----------------------------------

	QDir m_pluginsDir;
	QStringList m_pluginFileNames;

	//---- ONLY TEMP
	QWidget *m_pAlpha;
	QWidget *m_pColor;
	//----

	void loadPlugins();
	void populateSettingsDock(QObject *plugin, const QString &uuid );

	void produceToolButtons(QObject *plugin);
	void addToolButton(QObject *plugin, const QList<QUuid> &uuids,
					   const QStringList &texts, const QList<QIcon> &icons,
					   const char *member);
	void eraseLayout(QLayout *layout);

	void writeSettings();
	void readSettings();

	MdiChild* getMdiChild(QMdiSubWindow *window) const;
	QMdiSubWindow *getSubWindow() const;
	QAction* getActiveActionWindow() const;
	void initaiteDockMenu(void);
	void initiateActionWigets(void);
#ifndef QT_NO_DEBUG
	QMenu *m_debugMenu;
	QAction *m_repaintPaintArea;
#endif

public slots:
	void aboutPlugins();

private slots:
	void on_actionTabs_mode_triggered();
	void on_actionWindowed_mode_triggered();

	void showAbout();
	void showNewDialog();
	void createNew(QVariantMap);

	void subWindowActiv(QMdiSubWindow*);
	void changeWindowTitle(QString title);
	void changeWindowModified(bool _state);
	void setActiveSubWindow(QWidget*);
	void closeActive();
	void prepareCloseActive(MdiChild* child);
	void changeBrush();

	//-----------------
	void changeColor();
	void changeAlpha(int value);
	void changeAA(int value);
	void changeSize(int value);
	//-----------------

	void openFile();
	void save();
	void saveAs();
	void undo();
	void redo();
#ifndef QT_NO_DEBUG
	void repaintCurrent();
#endif
};

#endif // MAINWINDOW_H
