#include "newdialog.h"
#include "ui_newdialog.h"
#include "definitions.h"


#include <cmath>

#include <QIntValidator>
#include <QColor>
#include <QSettings>
#include <QInputDialog>
#include <QMessageBox>

#ifndef QT_NO_DEBUG
	#include <QDebug>
#endif

NewDialog::NewDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::NewDialog)
{
    ui->setupUi(this);

	//lock dialog size change
	this->setFixedSize(this->geometry().width(), this->geometry().height());
	//hide maximize button
	this->setWindowFlags(Qt::Dialog | Qt::WindowCloseButtonHint);

	static int sequenceNumber = 1;

	ui->lineEdit_name->setText( tr("Untitled%1").arg(sequenceNumber++) );

	connect(ui->pushButton, SIGNAL(clicked()),this, SLOT(myClose()));
	connect(ui->pushButton_2, SIGNAL(clicked()), this, SLOT(close()));
	connect(ui->pushButton_savePreset, SIGNAL(clicked()),
			this, SLOT(savePreset()));

	ui->comboBox_width->addItem(tr("pixels"));
	ui->comboBox_width->addItem(tr("inches"));
	ui->comboBox_width->addItem(tr("cm"));
	ui->comboBox_width->addItem(tr("mm"));

	ui->comboBox_height->addItem(ui->comboBox_width->itemText(0));
	ui->comboBox_height->addItem(ui->comboBox_width->itemText(1));
	ui->comboBox_height->addItem(ui->comboBox_width->itemText(2));
	ui->comboBox_height->addItem(ui->comboBox_width->itemText(3));

	ui->comboBox_resolution->addItem(tr("pixels/inch"));
	ui->comboBox_resolution->addItem(tr("pixels/cm"));

	ui->comboBox_background->addItem(tr("White"));
	ui->comboBox_background->addItem(tr("Custom Color<-not work, "
										"actually transparent"));
	ui->comboBox_background->addItem(tr("Transparent"));


	ui->comboBox_width->setCurrentIndex(0);
	ui->comboBox_height->setCurrentIndex(0);

	ui->comboBox_resolution->setCurrentIndex(0);
	ui->comboBox_background->setCurrentIndex(0);

	//FIXME
	//it should depend of measure
	ui->lineEdit_width->setValidator(new QIntValidator(1, 4096,
													   ui->lineEdit_width));
	ui->lineEdit_height->setValidator(new QIntValidator(1, 4096,
														ui->lineEdit_height));
	ui->lineEdit_resolution->setValidator(new QIntValidator(1, 4096,
														ui->lineEdit_height));

	connect(ui->comboBox_height, SIGNAL(currentIndexChanged(int)),
			ui->comboBox_width, SLOT(setCurrentIndex(int)));
	connect(ui->comboBox_width, SIGNAL(currentIndexChanged(int)),
			ui->comboBox_height, SLOT(setCurrentIndex(int)));
	connect(ui->comboBox_preset, SIGNAL(currentIndexChanged(int)),
			this, SLOT(applyPreset(int)));


	readSettings();
	readPresets();
}

NewDialog::~NewDialog(){
    delete ui;
}

void NewDialog::myClose(){
	QVariantMap returned;

	returned.insert("windowName",ui->lineEdit_name->text());


	int width = ui->lineEdit_width->text().toInt();
	int height = ui->lineEdit_height->text().toInt();
	int dpi = ui->lineEdit_resolution->text().toInt();

	if( ui->comboBox_resolution->currentIndex() == 1 ){
		dpi = float(dpi) * 2.54;
	}


	switch(ui->comboBox_width->currentIndex()){
	case 1://inches
		width = width * dpi;
		height = height * dpi;
		break;
	case 2://cm
		width = float( 10*width * dpi ) / 25.4;
		height = float( 10*height * dpi ) / 25.4;
		break;
	case 3://mm
		width = float( width * dpi ) / 25.4;
		height = float( height * dpi ) / 25.4;
		break;
	}


	returned.insert("width", width);
	returned.insert("height", height);

	returned.insert("width_m",ui->comboBox_width->currentIndex());
	returned.insert("height_m",ui->comboBox_height->currentIndex());

	returned.insert("dpi", dpi);
	returned.insert("dpi_m", ui->comboBox_resolution->currentIndex());


	if( ui->comboBox_background->currentIndex() == 0)
		returned.insert("background", QColor(255,255,255));
	else
		returned.insert("background", QColor(255,255,255,0));


	writeSettings();

	emit createNew(returned);

	this->close();
}

void NewDialog::writeSettings(){
	QSettings settings;

	settings.setIniCodec("UTF-8");
	settings.beginGroup("NewDocument");

	settings.setValue("width",ui->lineEdit_width->text());
	settings.setValue("height",ui->lineEdit_height->text());

	settings.setValue("width_m", ui->comboBox_width->currentIndex());
	settings.setValue("height_m", ui->comboBox_height->currentIndex());

	settings.setValue("dpi",ui->lineEdit_resolution->text());
	settings.setValue("dpi_m", ui->comboBox_resolution->currentIndex());

	settings.setValue("background", ui->comboBox_background->currentIndex());

	settings.endGroup();
}

void NewDialog::readSettings(){

	QSettings settings;

	settings.setIniCodec("UTF-8");
	settings.beginGroup("NewDocument");
	ui->lineEdit_width->setText( settings.value("width", 300).toString() );
	ui->lineEdit_height->setText( settings.value("height", 300).toString() );

	ui->comboBox_width->setCurrentIndex(
			settings.value("width_m", 0).toInt() );
	ui->comboBox_height->setCurrentIndex(
			settings.value("height_m", 0).toInt() );

	ui->lineEdit_resolution->setText( settings.value("dpi", 72).toString() );
	ui->comboBox_resolution->setCurrentIndex(
			settings.value("dpi_m", 0).toInt() );

	ui->comboBox_background->setCurrentIndex(
			settings.value("background", 0).toInt() );
	settings.endGroup();
}

bool NewDialog::checkPresetName(const QString &name){
	foreach(QVariantMap pos, m_presets){
		if( pos.value("name").toString().toLower() == name.toLower() )
			return true;
	}
	return false;
}

void NewDialog::readPresets(){
	XMLParser xml;

	ui->comboBox_preset->addItem(tr("Custom"));
	ui->comboBox_preset->setCurrentIndex(0);
	if( xml.readPresets(m_presets) ){
		foreach(QVariantMap pos, m_presets){
			ui->comboBox_preset->addItem(
					pos.value("name","defaultName").toString());
		}
	}
}

void NewDialog::applyPreset(int pos){
	if( pos == 0 )
		readSettings();
	else{
		QVariantMap preset = m_presets.at(pos-1);
		ui->lineEdit_width->setText(
				preset.value("width", 300).toString() );
		ui->lineEdit_height->setText(
				preset.value("height", 300).toString() );

		ui->comboBox_width->setCurrentIndex(
				preset.value("width_metric", 0).toInt() );
		ui->comboBox_height->setCurrentIndex(
				preset.value("height_metric", 0).toInt() );

		ui->lineEdit_resolution->setText(
				preset.value("dpi", 72).toString() );
		ui->comboBox_resolution->setCurrentIndex(
				preset.value("dpi_metric", 0).toInt() );

		ui->comboBox_background->setCurrentIndex(
				preset.value("background_metric", 0).toInt() );
	}
}

void NewDialog::savePreset(){
	QVariantMap data;
	data.insert("width", ui->lineEdit_width->text());
	data.insert("height", ui->lineEdit_height->text());

	data.insert("width_metric", ui->comboBox_width->currentIndex());
	data.insert("height_metric", ui->comboBox_height->currentIndex());

	data.insert("dpi", ui->lineEdit_resolution->text());
	data.insert("dpi_metric", ui->comboBox_resolution->currentIndex());

	data.insert("background_metric",ui->comboBox_background->currentIndex());

	bool ok;
	QString textName;
	do{
		 textName = QInputDialog::getText(this, tr("Type preset name"),
									  tr("Preset name:"),
									  QLineEdit::Normal,
									  tr("DefaultName"), &ok,
									  Qt::Dialog | Qt::WindowCloseButtonHint );
		 if( !ok )
			 return;

		 if( textName.isEmpty() ){
			 QMessageBox::information(this, tr("Information"),
									  tr("Name cannot be empty"));
		 }else{
			 if( checkPresetName(textName) ){
				 QMessageBox::warning(this, tr("Warning"),
										  tr("Preset with this name exists"));
				 textName.clear();
			 }
		 }


	}while ( textName.isEmpty() );




	data.insert("name",textName);

	XMLParser xml;
	if( xml.savePreset(data) ){
		ui->comboBox_preset->addItem(textName);
		m_presets.append(data);
	}else{
		QMessageBox::warning(this, tr("Warning"), tr("Preset not saved"));
	}

}
